<?php
/**
 * @author Richard Fussenegger <fleshgrinder@users.noreply.github.com>
 * @copyright 2015 Richard Fussenegger
 * @license MIT
 */

namespace Fleshgrinder\Core;

/**
 * Defines the comparable interface.
 *
 * Requires implementing classes to provide a compare method, comparable to the method in Java but (of course) without
 * automatic invocation. However, this [will be supported in PHP 7](https://wiki.php.net/rfc/comparable). Please refer
 * to the RFC for an in-depth discussion of this interface.
 */
interface Comparable {

    /**
     * Compare this object with the specified value for order.
     *
     * The implementor must ensure `gmp_sign($x->compareTo($y)) === -gmp_sign($y->compareTo($x))` for all `$x` and `$y`.
     * (This implies that `$x->compareTo($y)` must throw an exception if `$y->compareTo($x)` throws an exception.)
     *
     * The implementor must also ensure that the relation is transitive: `$x->compareTo($y) > 0 && $y->compareTo($z) > 0`
     * implies `$x->compareTo($z) > 0`.
     *
     * Finally, the implementor must ensure that `$x->compareTo($y) === 0` implies that `gmp_sign($x->compareTo($z)) ===
     * gmp_sign($y->compareTo($z))`, for all `$z`.
     *
     * It is strongly recommended, but not strictly required that `($x->compareTo($y) === 0) === ($x->equals($y))`.
     * Generally speaking, any class that implements the {@see Comparable} interface and violates this condition should
     * clearly indicate this fact. The recommended language is "Note: this class has a natural ordering that is
     * inconsistent with equals."
     *
     * @param mixed $value
     *     Value to compare to.
     * @return integer
     *     Negative integer, zero, or a positive integer as this object is less than, equal to, or greater than the
     *     specified value.
     * @throws UncomparableException
     *     If the implementor is unable to compare the given value with itself.
     */
    public function compareTo($value);

}
